

/* Includes ------------------------------------------------------------------*/
#include "lwip/opt.h"
#include "lwip/arch.h"
#include "lwip/api.h"
#include "string.h"
#include "tcp_echoclient-netconn.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define TCPECHOCLIENT_THREAD_PRIO    ( tskIDLE_PRIORITY + 2UL )
#define TCPECHOCLIENT_THREAD_STACKSIZE  200
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/


/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief serve tcp connection
  * @param conn: pointer on connection structure
  * @retval None
  */
void tcp_client_serve(struct netconn *conn)
{
    struct netbuf *inbuf;
    char* buf;
    u16_t buflen;

    while(1) {
        printf("Wait for TCP data       ...");

        /* Read the data from the port, blocking if nothing yet there.
         We assume the request is in one netbuf */
        if(netconn_recv(conn,&inbuf) != ERR_OK) {
            netbuf_delete(inbuf);
            break;
        }

        printf(" [OK] ...\n");
        if(inbuf != NULL) {
            if(netconn_err(conn) == ERR_OK) {
                netbuf_data(inbuf, (void**)&buf, &buflen);
                netconn_write(conn, (const unsigned char*)buf, (size_t)buflen, NETCONN_NOCOPY);
            }
        }

        /* Delete the buffer (netconn_recv gives us ownership,
         so we have to make sure to deallocate the buffer) */
        netbuf_delete(inbuf);

        printf(" [OK] ...\n");
    }
}

/**
  * @brief  TCP echo client thread
  * @param arg pointer on argument(not used here)
  * @retval None
  */
static void tcp_echoclient_netconn_thread(void *arg)
{
    struct netconn *conn;
    err_t err;
    ip_addr_t ipaddr;

    IP4_ADDR(&ipaddr, 192,168,0,3);

    while(1) {
        /* Create a new TCP connection handle */
        conn = netconn_new(NETCONN_TCP);

        if(conn!= NULL) {
            /* Bind to port 80 with default IP address */
            err = netconn_bind(conn, NULL, 80);

            if(err == ERR_OK) {
                err = netconn_connect(conn, &ipaddr, 80);   /*connect to server*/

                if(err == ERR_OK) {
                    printf("Connect server succeed ! \n");
                    /*Send a message to server*/
                    netconn_write(conn, (const unsigned char*)"Hello server !", (size_t)strlen("Hello server !"), NETCONN_NOCOPY);
                    tcp_client_serve(conn);

                } else {
                    printf("Connect server fail ! \n");
                }
            } else {
                printf("can not bind netconn");
            }
            netconn_delete(conn);
        } else {
            printf("can not create netconn");
        }

        vTaskDelay(50 / portTICK_RATE_MS);
    }
}

/**
  * @brief  Initialize the TCP client (start its thread)
  * @param  none
  * @retval None
  */
void tcp_echoclient_netconn_init()
{
    sys_thread_new("TCPECHO", tcp_echoclient_netconn_thread, NULL, TCPECHOCLIENT_THREAD_STACKSIZE, TCPECHOCLIENT_THREAD_PRIO);
}

